<?php /*

 ----> Dreadnaut's File List 0.1 --------------------------------------

 Contacts:
  - http://dreadnaut.altervista.org/
  - dreadnaut@altervista.org

 Released under the 
   Creative Commons Attribution-NonCommercial-ShareAlike 3.0 License.
   [ http://creativecommons.org/licenses/by-nc-sa/3.0/ ]
   

 ----> Standard functions: --------------------------------------------

 > $dfl = new DFileList( [ $path ] )
 creates a new File List for the specified folder. Current folder is
  used if path is omitted or empty



 > setType($type, $patterns[], [ $directory ] )
 adds a file type, defined by one or more wildcard patterns. If
  directory is non-zero, the type is defined for directories
  
 $dfl->setType( 'image',  array( '*.png', '*.jpg' );
 $dfl->setType( 'folder', '*', DFL_DIRECTORY );
 
 
 
 > setHidden( $patterns[] )
 adds one or more groups of files or directories to hide from listing
 
 $dfl->setHidden( array('index.php', 'dFileList.inc.php') );
 
 
 
 > $l = getList( [ $flags ] )
 retrieves a list of files. Possible (additive) flags are:
  DFL_DIRECTORY - to show directories
  DFL_DIRFIRST  - to show directories at the beginning of the list

 the resulting array includes a "total" record containing the number of
 files and directories read and their size, and a "files" list of all
 the files in the directory, filtered and sorted.
 
 $list = $dfl->getList( DFL_DIRECTORY | DFL_DIRFIRST );



 > $s = renderList( $list )
 renders a list of files (as retrieved with getList) into a string,
 applying templates, field transformations and file processing if
 specified.

 echo $dfl->renderList( $list );


 ----> Advanced functions: --------------------------------------------

 > setTemplate( $defaulttemplate )
  sets the default template.
  
  $dfl->setTemplate( '<li>%%name%% - %%size%%</li>' );



 > setTemplate( $type, $template )
  sets the template for the specified $type

  $dfl->setTemplate( 'image' => '<li>%%name%% - %%size%%</li>' );



 > setTemplate( $templates[] )
  sets a list of templates. Array is in the form:
   array( 'type' => 'template', ... )

  $dfl->setTemplate( array(
   'image' => '<li>%%name%% - %%size%% is a picture</li>',
   'text'  => '<li>%%name%% - %%size%% is text</li>'
   ) );



 > setTransform($elem, $function)
  sets a transformation function for a particular field. The function
  will receive a string with the field content and must return a string
  which will be used in the template instead of the original field.
  
  function makeKB( $input ) {
    return $input / 1024 . 'KB';
  }
  
  $dfl->setTransform( 'size', 'makeKB' );



 > setProcess($type, $function)
  sets a preprocess function for a particular filetype. The function
  will receive a reference to a file record, and is allowed to modify
  or add fields to it

  function countFileInDir(&$file) {
    if ($file['type'] = 'folder')
      $file['count'] = count( glob($file['fullname'] . '/*') );
  }

  $dfl->setType('folder', '*', DFL_DIRECTORY);
  $dfl->setTemplate('folder', '<li>%%name%% contains %%count%% files</li>');
  $dfl->setProcess('folder', 'countFilesInDir');
  
  will count and output the number of files in every subdirectory.
  
     

 > about( [$lang] )
  return an "about" string it would be nice to see at the bottom of your
  page, if you are going to use this script :)  Optional language parameter,
  but only a few translations are available:  'it' and 'en' (default).
  
  echo $dfl->about('it');

 ----> File record ----------------------------------------------------
  A file record is an array including the following fields:

   'fullname' => $file
   'name'     => basename($file)
   'size'     => filesize($file)
   'time'     => filemtime($file)
   'perms'    => fileperms($file)
   'type'     => <set according to type patterns>

  If a transformation function is present, the field will go through it,
  and the will be substituted for the corrispondent %%field%% string in
  the template for the file 'type'.

*/




// --------------------------------------------------------------------
// ----------------- Here there be dragons ----------------------------
// --------------------------------------------------------------------

// constants for DFileList::getList() 

define('DFL_DIRECTORY', 1); // show directories
define('DFL_DIRFIRST',  2); // show directories at the beginning of the list

define('DFL_DEFAULT_TEMPLATE', 
 '&raquo; <a href="%%fullname%%"><strong>%%name%%</strong></a> is a <em>%%type%%</em> of size %%size%% bytes<br/>');

// define fnmatch is the function is missing (on windows or php < 4.3.0)
if (!function_exists('fnmatch')) {
  function fnmatch($pattern, $string) {
    return @preg_match(
      '/^' . strtr(addcslashes($pattern, '/\\.+^$(){}=!<>|'),
        array('*' => '.*', '?' => '.?')) . '$/i', $string
    );
  }
}


// ------------------ DFileList class ---------------------------------
class DFileList {

  var $aDirTypes  = array();
  var $aTypes     = array();
  var $aTemplates = array();
  var $aHide      = array();
  var $aTransform = array();
  var $aProcess   = array();

  var $pRoot  = '';

  var $iFileCount = 0;
  var $iDirCount  = 0;
  var $iFileSize  = 0;


// ----> DFileList() --------------------------------------------------
  function DFileList($path = '') {
    $this->pRoot = $path;  
  }


// ----> setType() ----------------------------------------------------
  function setType($type, $patterns = array(), $directory = 0) {
    if (!is_array($patterns))
      $patterns = array($patterns);
    if (count($patterns) == 0)
      return false;

    if ($directory)
      $list =& $this->aDirTypes;
    else
      $list =& $this->aTypes;

    $first = array_shift($patterns);
    $list[$first] = $type;
    $count = 1;

    foreach($patterns as $pattern) {
      $list[$pattern] =& $list[$first];
      $count += 1;
    }

    return $count;
  }


// ----> setHidden() --------------------------------------------------
  function setHidden($patterns = array()) {
    if (!is_array($patterns))
      $patterns = array($patterns);
    if (count($patterns) == 0)
      return false;

    $count = 0;

    foreach($patterns as $pattern) {
      $this->aHide[] = $pattern;
      $count += 1;
    }

    return $count;
  }


// ----> getList() ----------------------------------------------------
  function getList($flags = 0) {
    $totals = array(
       'dir' => 0,
       'files' => 0,
       'size' => 0
      );
    $files = array();
    $dirs  = array();

    $list = glob($this->pRoot . '*');

    foreach($list as $file) {
      $rec = array(
       'fullname' => $file,
       'name'     => basename($file),
       'size'     => filesize($file),
       'time'     => filemtime($file),
       'perms'    => fileperms($file),
       'type'     => ''
      );

      $toHide = false;
      foreach($this->aHide as $pattern)
        if (fnmatch($pattern, $rec['name'])) {
          $toHide = true;
          break;
        }
      if ($toHide) continue;

      if (filetype($file) == 'dir') {

        foreach($this->aDirTypes as $pattern => $type)
          if (fnmatch($pattern, $rec['name'])) {
            $rec['type'] = $type;
            break;
          }

        if ($flags & DFL_DIRECTORY) {
          $totals['dir'] += 1;

          if ($flags & DFL_DIRFIRST)
            $dirs[] = $rec;
          else
            $files[] = $rec;
        }

      } else {

        foreach($this->aTypes as $pattern => $type)
          if (fnmatch($pattern, $rec['name'])) {
            $rec['type'] = $type;
            break;
          }

        $files[] = $rec;
        $totals['files'] += 1;
        $totals['size']  += $rec['size'];

      }

    }
    
    if ($flags & DFL_DIRFIRST)
      $files = array_merge($dirs, $files);

    return array(
     'total' => $totals,
     'files' => $files
     );
  }


// ----> setTransform() -----------------------------------------------
  function setTransform($elem, $function) {
    if (!function_exists($function))
      return false;

    $this->aTransform[$elem] = $function;
    return true;
  }
  
// ----> setProcess() -------------------------------------------------
  function setProcess($type, $function) {
    if (!function_exists($function))
      return false;

    $this->aProcess[$type] = $function;
    return true;
  }


// ----> setTemplate() ------------------------------------------------
  function setTemplate($templates, $template = '') {
    $count = 0;

    if (is_array($templates)) {

      if (count($templates) == 0)
        return false;

      $count = 0;
      foreach($templates as $type => $template) {
        $this->aTemplates[$type] = $template;
        $count += 1;
      }

      return $count;

    } else {
      if ($template == '') {
        $this->aTemplates['*'] = $templates;
      } else {
        $this->aTemplates[$templates] = $template;
      }

      return 1;
    }

  }


// ----> renderList() -------------------------------------------------
  function renderList($list = array()) {
    if (!isset($list['files']))
      return false;

    $output = '';
    foreach($list['files'] as $file)
      if (is_array($file)) {

        $elem = DFL_DEFAULT_TEMPLATE;

        foreach($this->aTemplates as $pattern => $template)
          if (fnmatch($pattern, $file['type'])) {
            $elem = $template;
            break;
          }

        foreach($this->aProcess as $pattern => $process)
          if (fnmatch($pattern, $file['type'])) {
            eval($process . '($file);');
            break;
          }

        foreach($file as $key => $value) {
          if (isset($this->aTransform[$key]))
            $value = call_user_func($this->aTransform[$key], $value, $file['fullname']);
          $elem = str_replace("%%$key%%", $value, $elem);
        }

        $output .= $elem;
      }

    return $output;

  }
  
  
// ----> about() ------------------------------------------------------
  function about($lang = '') {
    switch ($lang) {
     case 'it':
      return 'Generato con <a href="http://dreadnaut.altervista.org/">Dreadnaut\'s File List</a>.';
     default:
      return 'Generated with <a href="http://dreadnaut.altervista.org/">Dreadnaut\'s File List</a>.';
    }
        
  }

}

// -eof-
?>